# Seongho Bak, Taewon Kim, 2023

import omni.isaac.core.tasks as tasks
from view_opt_task import ViewOptimizeTask
from ur5e_handeye import UR5eHandeye
from omni.isaac.core.utils.prims import is_prim_path_valid
from omni.isaac.core.utils.string import find_unique_string_name
from typing import Optional
import numpy as np


class ViewOptimizeRobot(ViewOptimizeTask):
    """[summary]
        made by seongho bak.
        modified 'FollowTarget' class
        modified from '~/.local/share/ov/pkg/isaac_sim-2022.2.0/exts/omni.isaac.franka/omni/isaac/franka/tasks/follow_target.py'

        Args:
            name (str, optional): [description]. Defaults to "ur5e_follow_target".
            target_prim_path (Optional[str], optional): [description]. Defaults to None.
            target_name (Optional[str], optional): [description]. Defaults to None.
            target_position (Optional[np.ndarray], optional): [description]. Defaults to None.
            target_orientation (Optional[np.ndarray], optional): [description]. Defaults to None.
            offset (Optional[np.ndarray], optional): [description]. Defaults to None.
            franka_prim_path (Optional[str], optional): [description]. Defaults to None.
            franka_robot_name (Optional[str], optional): [description]. Defaults to None.
        """

    def __init__(
        self,
        name: str = "ur5e_follow_target",
        target_prim_path: Optional[str] = None,
        target_name: Optional[str] = None,
        target_position: Optional[np.ndarray] = None,
        target_orientation: Optional[np.ndarray] = None,
        offset: Optional[np.ndarray] = None,
        ur5e_prim_path: Optional[str] = None,
        ur5e_robot_name: Optional[str] = None,
        ur5e_usd_path: Optional[str] = None,
    ) -> None:
        ViewOptimizeTask.__init__(
            self,
            name=name,
            target_prim_path=target_prim_path,
            target_name=target_name,
            target_position=target_position,
            target_orientation=target_orientation,
            offset=offset,
        )
        self._ur5e_prim_path = ur5e_prim_path
        self._ur5e_robot_name = ur5e_robot_name
        self._ur5e_usd_path = ur5e_usd_path
        return

    def set_robot(self) -> UR5eHandeye:
        """[summary]

        Returns:
            Franka: [description]
        """
        if self._ur5e_prim_path is None:
            self._ur5e_prim_path = find_unique_string_name(
                initial_name="/World/ur5e", is_unique_fn=lambda x: not is_prim_path_valid(x)
            )
        if self._ur5e_robot_name is None:
            self._ur5e_robot_name = find_unique_string_name(
                initial_name="ur5e", is_unique_fn=lambda x: not self.scene.object_exists(x)
            )
        if self._ur5e_usd_path is not None:
            return UR5eHandeye(prim_path=self._ur5e_prim_path,
                               name=self._ur5e_robot_name,
                               usd_path=self._ur5e_usd_path)  ### BSH

        return UR5eHandeye(prim_path=self._ur5e_prim_path, name=self._ur5e_robot_name)    ### BSH
