# Seongho Bak, Taewon Kim, 2023
"""
made by seongho bak
modified follow target example
modified from '~/.local/share/ov/pkg/isaac_sim-2022.2.0/standalone_examples/api/omni.isaac.franka/follow_target_with_ik.py'
"""

from omni.isaac.kit import SimulationApp
simulation_app = SimulationApp({"headless": False})

from view_opt_ur5e import ViewOptimizeRobot
from omni.isaac.core import World
from kinematics_solver import KinematicsSolver
import omni.isaac.core.utils.numpy.rotations as rot_utils
from omni.isaac.examples.base_sample import BaseSample
from omni.isaac.core.objects import DynamicCuboid
from omni.isaac.core.objects import DynamicCone
from omni.isaac.core.objects import DynamicCapsule
from omni.isaac.core.objects import DynamicCylinder
from omni.isaac.core.objects import DynamicSphere
from omni.isaac.core.utils.stage import get_stage_units
from omni.isaac.core.utils.rotations import euler_angles_to_quat, quat_to_euler_angles
from omni.isaac.core.utils.semantics import add_update_semantics

from omni.kit.viewport_legacy import get_viewport_interface
from omni.kit.viewport.utility import get_active_viewport, get_active_viewport_camera_path

import carb

import os, cv2
import numpy as np

class AddObjects(BaseSample):
    def __init__(self) -> None:
        super().__init__()
        # self.world = World(stage_units_in_meters=1.0)
        # self.robot_asset_path = assets
        return

    def add_primitives(self, world):
        cube = DynamicCuboid(
                    prim_path="/new_cube_1",
                    name="cube_1",
                    position=np.array([-1, -0.3, 2.0]),
                    scale=np.array([0.15, 0.15, 0.15]),
                    size=1.0,
                    color=np.array([255, 0, 0]),
                    )
        cylinder = DynamicCylinder(
                    prim_path="/new_cylinder_1",
                    name="cylinder_1",
                    position=np.array([-1, 0, 2.0]),
                    scale=np.array([0.1, 0.1, 0.2]),
                    color=np.array([0, 255, 0]),
                    )
        cone = DynamicCone(
                    prim_path="/new_cone_1",
                    name="cone_1",
                    position=np.array([-1, 0.3, 2.0]),
                    scale=np.array([0.1, 0.1, 0.2]),
                    color=np.array([0, 0, 255]),
                    )
        capsule = DynamicCapsule(
                    prim_path="/new_capsule_1",
                    name="capsule_1",
                    position=np.array([-1.4, 0.15, 2.0]),
                    orientation=rot_utils.euler_angles_to_quats(np.array([0, 90, -45]), degrees=True),
                    scale=np.array([0.08, 0.08, 0.12]),
                    color=np.array([255, 255, 0]),
                    )
        sphere = DynamicSphere(
                    prim_path="/new_sphere_1",
                    name="sphere_1",
                    position=np.array([-1.4, -0.15, 2.0]),
                    scale=np.array([0.1, 0.1, 0.1]),
                    color=np.array([255, 0, 255]),
                    )

        object_cube = world.scene.add(cube)
        object_cube.enable_rigid_body_physics()
        object_cylinder = world.scene.add(cylinder)
        object_cylinder.enable_rigid_body_physics()
        object_cone = world.scene.add(cone)
        object_cone.enable_rigid_body_physics()
        object_capsule = world.scene.add(capsule)
        object_capsule.enable_rigid_body_physics()
        object_sphere = world.scene.add(sphere)
        object_sphere.enable_rigid_body_physics()

        return [cube, cylinder, cone, capsule, sphere]


my_world = World(stage_units_in_meters=1.0)

working_dir = os.path.dirname(os.path.realpath(__file__))
# ur5e_usd = os.path.join(working_dir, "ur5e_handeye_gripper_upright.usd")
# ur5e_usd = os.path.join(working_dir, "ur5e_handeye_gripper_checkdepth.usd")
ur5e_usd = os.path.join(working_dir, "ur5e_handeye_gripper.usd")
# ur5e_usd = "/home/bak/.local/share/ov/pkg/isaac_sim-2022.2.0/standalone_examples/user_example/ur5e_handeye_gripper.usd"
if os.path.isfile(ur5e_usd):
    pass
else:
    raise Exception(f"{ur5e_usd} not found")

target_position = np.array([-0.6, 0, 0.5]) / get_stage_units()
target_orientation = euler_angles_to_quat(np.array([-np.pi/2, np.pi/4, np.pi]))
my_task = ViewOptimizeRobot(name="view_optimize_task", ur5e_usd_path=ur5e_usd,
                            target_position=target_position,
                            target_orientation=target_orientation)

my_world.add_task(my_task)
add_objects = AddObjects()
objects_list = add_objects.add_primitives(my_world)

# add semantics
for object_shape in objects_list:
    print(object_shape)
    add_update_semantics(prim=object_shape.prim, semantic_label=object_shape.name)

my_world.reset()

task_params = my_world.get_task("view_optimize_task").get_params()
ur5e_name = task_params["robot_name"]["value"]
target_name = task_params["target_name"]["value"]
my_ur5e = my_world.scene.get_object(ur5e_name)
my_controller = KinematicsSolver(my_ur5e, end_effector_frame_name="flange")

rgb_cam = my_ur5e.rgb_cam
# depth_cam = my_ur5e.depth_cam

# depth_cam.initialize()
rgb_cam.initialize()

# depth_cam.add_distance_to_camera_to_frame()
rgb_cam.add_instance_id_segmentation_to_frame()
rgb_cam.add_instance_segmentation_to_frame()


viewport = get_active_viewport()
# viewport.set_active_camera('/World/ur5e/realsense/rgb')
# viewport.set_active_camera('/OmniverseKit_Persp')

image_save_path = f"{os.path.expanduser('~')}/isaac_sim_image"
if not os.path.exists(image_save_path):
    os.makedirs(image_save_path)

articulation_controller = my_ur5e.get_articulation_controller()

i = 0
while simulation_app.is_running():
    my_world.step(render=True)
    if my_world.is_playing():
        if my_world.current_time_step_index == 0:
            my_world.reset()
        observations = my_world.get_observations()
        actions, succ = my_controller.compute_inverse_kinematics(
            target_position=observations[target_name]["position"],
            target_orientation=observations[target_name]["orientation"],
        )
        
        print(type(observations[target_name]["orientation"]))
        # array([-0.6,  0. ,  0.5], dtype=float32)
        # array([-0.27059805, -0.27059805, -0.6532815 ,  0.6532815 ], dtype=float32)
        angle = quat_to_euler_angles(observations[target_name]["orientation"])
        print(type(angle))
        print(angle[0]*180/3.1415, angle[1]*180/3.1415, angle[2]*180/3.1415)

        ''' save image '''
        # if i % 20 == 0 and i>300:
        #     rgb_image = rgb_cam.get_rgba()[:, :, :3]
        #     rgb_image = cv2.cvtColor(rgb_image, cv2.COLOR_BGR2RGB)
        #     depth_image = depth_cam.get_current_frame()["distance_to_camera"]
        #     try:
        #         instance_image = rgb_cam.get_current_frame()["instance_segmentation"]["data"]
        #         instance_image = np.array(instance_image, dtype=np.uint8)
        #         # instance_id_image = rgb_cam.get_current_frame()["instance_id_segmentation"]["data"]
        #         # instance_id_image = np.array(instance_id_image, dtype=np.uint8)
        #     except:
        #         print('Passing create instance ')
        #         pass

            # rgb_save_path = os.path.join(image_save_path, f"rgb_camera_{i}.png")
            # depth_save_path = os.path.join(image_save_path, f"depth_camera_{i}.png")
            # instance_save_path = os.path.join(image_save_path, f"instance_{i}.png")
            # # instance_id_save_path = os.path.join(image_save_path, f"instance_id_{i}.png")

            # try:
            #     cv2.imwrite(rgb_save_path, rgb_image)
            #     cv2.imwrite(depth_save_path, depth_image*100)
            #     cv2.imwrite(instance_save_path, instance_image)
            #     # cv2.imwrite(instance_id_save_path, instance_id_image)
            # except:
            #     print('Pass')
            #     pass
        i += 1


        if succ:
            articulation_controller.apply_action(actions)
        else:
            carb.log_warn("IK did not converge to a solution.  No action is being taken.")

simulation_app.close()
